//+------------------------------------------------------------------+
//|                                                       exchng.mq4 |
//|                                           Copyright  2009, alsu |
//|                                                 alsufx@gmail.com |
//+------------------------------------------------------------------+
#property copyright "Copyright  2009, alsu"
#property link      "alsufx@gmail.com"
#property library

#import "exchng.dll"
int GetPtr(double &a[]);
double GetValue(int pointer,int i);
void SetValue(int pointer,int i,double value);
#import
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void RegisterBuffer(double &Buffer[],string name,int mode)
  {
   UnregisterBuffer(Buffer);                    //first of all, let's delete the variable to be on the safe side

   int direction=0;
   if(ArrayGetAsSeries(Buffer)) direction=1;    //set indexing direction flag

   name=name+"#"+mode+"#"+Symbol()+"#"+Period()+"#"+ArraySize(Buffer)+"#"+direction;
   int ptr=GetPtr(Buffer);                //take out the pointer
   if(ptr==0) return;

   MathSrand(ptr);                            //it is very convenient to use the pointer value (not time) to initialize pseudo-random number generator

   while(true)
     {
      int rnd=MathRand();
      if(!GlobalVariableCheck(name+"#"+rnd))    //provide uniqueness - I believe, no one will want to 
        {                                         //explore more than RAND_MAX buffers:)
         name=name+"#"+rnd;
         GlobalVariableSet(name,ptr);         //write into globals
         break;
        }
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void UnregisterBuffer(double &Buffer[])
  {
   int ptr=GetPtr(Buffer);                  //unregister at the buffer's actual address
   if(ptr==0) return;

   int gt=GlobalVariablesTotal();
   int i;
   for(i=gt-1;i>=0;i--) //search all global variables 
     {                                            //and delete our buffer from all possible places
      string name=GlobalVariableName(i);
      if(GlobalVariableGet(name)==ptr)
         GlobalVariableDel(name);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int FindBuffers(string name,int mode,string symbol,int period,string &buffers[])
  {
   int count=0;
   int i;
   bool found;
   string name_i;
   string descriptor[];
   int gt=GlobalVariablesTotal();

   StringTrimLeft(name);                                    //reduce the string by deleting extra spaces
   StringTrimRight(name);

   ArrayResize(buffers,count);                              //reset the size

   for(i=gt-1;i>=0;i--)
     {
      found=true;
      name_i=GlobalVariableName(i);

      StringExplode(name_i,"#",descriptor);                 //divide the string into fields
      if(ArraySize(descriptor)>=4)
        {
         if(StringFind(descriptor[0],name)<0 && name!="") found=false;      //check if each field matches the search criteria
         if(StrToInteger(descriptor[1])!=mode && mode>=0) found=false;
         if(descriptor[2]!=symbol && symbol!="") found=false;
         if(StrToInteger(descriptor[3])!=period && period>0) found=false;

         if(found)
           {
            count++;                                           //all conditions are met, adding to the list
            ArrayResize(buffers,count);
            buffers[count-1]=name_i;
           }
        }
        else Print ("Error descriptor","i= ",i," " ,name_i);
     }

   return(count);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double GetIndicatorValue(string descriptor,int shift)
  {
   int ptr;
   string fields[];
   int size,direction;
   if(GlobalVariableCheck(descriptor)>0) //make sure the descriptor is valid
     {
      ptr=GlobalVariableGet(descriptor);                  //get the pointer value
      if(ptr!=0)
        {
         StringExplode(descriptor,"#",fields);              //divide the descriptor name to the fields
         size=StrToInteger(fields[4]);                                  //we need the current size of the array
         direction=StrToInteger(fields[5]);
         if(direction==1) shift=size-1-shift;               //if indexing is performed from the end
         if(shift>=0&&shift<size)                           //checking the index correctness to avoid the terminal crash
            return(GetValue(MathAbs(ptr),shift));           //done, returning the value
        }
     }
   return(EMPTY_VALUE);                                     //otherwise...
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool SetIndicatorValue(string descriptor,int shift,double value)
  {
   int ptr;
   string fields[];
   int size,direction;
   if(GlobalVariableCheck(descriptor)>0) //make sure the descriptor is valid
     {
      ptr=GlobalVariableGet(descriptor);                  //get the pointer value
      if(ptr!=0)
        {
         StringExplode(descriptor,"#",fields);              //divide the descriptor name to the fields
         size=StrToInteger(fields[4]);                                  //we need the current size of the array
         direction=StrToInteger(fields[5]);
         if(direction==1) shift=size-1-shift;               //if indexing is performed from the end
         if(shift>=0&&shift<size)                           //checking the index correctness to avoid the terminal crash
           {
            SetValue(MathAbs(ptr),shift,value);
            return(true);
           }
        }
     }
   return(false);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void StringExplode(string s,string separator,string &result[])
  {
   int i,pos;
   ArrayResize(result,1);

   pos=StringFind(s,separator);
   if(pos<0) {result[0]=s;return;}

   for(i=0;;i++)
     {
      pos=StringFind(s,separator);
      if(pos>=0)
        {
         result[i]=StringSubstr(s,0,pos);
         s=StringSubstr(s,pos+StringLen(separator));
        }
      else break;
      ArrayResize(result,ArraySize(result)+1);
     }
  }
//+------------------------------------------------------------------+
